-- ==============================================================================================
-- INTERNAL VARIABLES FOR MISC USE
-- ==============================================================================================

ROOT                = "Zelda 3 GBA Translation Comparison/"
MAIN_PAGE_NAME      = "main.html"
SECOND_PAGE_NAME    = "bottom.html"
PROJECT_TITLE       = "4-Way Zelda: LttP Text & Translation Comparison"
DO_AUTO_SAVES		= false -- no auto save functionality with this plugin :(
DO_AUTO_SCREENSHOTS = false -- I wanted to implement auto screenshots, but the game's text system loops during WAIT control codes in a way that makes it not very feasible :(
DO_AUTO_SCRIPTSTATS	= false
DATA_ROOT           = "Sidebars/" .. ROOT .. "data/"
SCRIPT_STATUS_FILE  = "_ScriptStatus.txt"
SCRIPT_PERCENT_FILE = "_ScriptPercent.txt"
SCRIPT_TALLY_FILE   = "_ScriptFinalTally.js"

totalLines = 463
seenLines  = 0
lineStatus = {}

seenSprite = {}

initializedBefore = false
MAIN_WINDOW_ID	  = 0
SECOND_WINDOW_ID  = -1

-- ==============================================================================================
-- ==============================================================================================
-- This runs once before doLogic ever runs
function init()

	-- this sets up a table we use for identifying the current enemies on the screen
	initializeSeenSprites()
	
	-- there's no way to delete windows so we don't want to accidentally create more of this window if we reset the game
	if initializedBefore == false then
		SECOND_WINDOW_ID = createWindow(960, 224, "Bottom Bar") -- SECOND_WINDOW_ID obtains a unique ID for this window, MAIN_WINDOW_ID should always be 0
		initializedBefore = true
	end
	
	resizev2(MAIN_WINDOW_ID, 540, 810)
	loadHTMLv2(MAIN_WINDOW_ID, ROOT .. MAIN_PAGE_NAME)	
	loadHTMLv2(SECOND_WINDOW_ID, ROOT .. SECOND_PAGE_NAME)
	
	if DO_AUTO_SCRIPTSTATS == true then
		initScriptStatus()
		loadScriptStatus()
		updateScriptPercentageSeen()
	end
end


-- ==============================================================================================
-- ==============================================================================================
-- This handles messages sent by the emulator, like if the player presses certain buttons or
-- other events happen within the emulator itself
function getMessage(str)
	if str == "resetgame" then
		init()
	elseif str == "loadstate" then
		loadHTMLv2(MAIN_WINDOW_ID, ROOT .. MAIN_PAGE_NAME)
		callJSFunctionv2(MAIN_WINDOW_ID, "clearAllData")
		callJSFunctionv2(SECOND_WINDOW_ID, "clearAllData")
	end
end



-- GAME IS ABOUT TO LOAD A LINE OF TEXT
function PC_81285DE()
	local lineID = getMemory(0x3003D18, 2)
	displayScriptText(lineID, "script")
end


-- GAME IS PROBABLY CLOSING A TEXT WINDOW
function PC_8128C66()
	callJSFunctionv2(MAIN_WINDOW_ID, "closeScriptDiv")
end


-- inside sprites being loaded
function PC_80CC5C0()
	initializeSeenSprites()
	callJSFunctionv2(SECOND_WINDOW_ID, "clearEnemyList")
	--local slotID = getR0() - 0x3003222
	--local spriteID = getR4()
	--callJSFunction_Int2("appendEnemy", slotID, spriteID)
end


--[[ outside sprites being loaded
function PC_80CA8A4()
end]]--


currScreenID = -1
lastScreenID = -1
currRoomID = -1
lastRoomID = -1
currGameMode = -1
lastGameMode = -1
frameCount = 0
subMode = 0
function doLogicFrame()

	-- let's check to see if we've changed overworld screens or gone inside or outside
	currGameMode = getCurrentGameMode()
	if currGameMode ~= 7 and currGameMode ~= 9 then
		lastGameMode = currGameMode
		return
	end
		
	-- if we've changed modes, we might need to clear the list of recently seen enemies
	if currGameMode ~= lastGameMode and ((currGameMode == 7 and lastGameMode == 9) or (currGameMode == 9 and lastGameMode == 7)) then
		-- if Link is controllable again (like after a room transition) then let's clear the list
		subMode = getMemory(0x3002BDD, 1)
		if subMode == 0 then
			initializeSeenSprites()
			callJSFunctionv2(SECOND_WINDOW_ID, "clearEnemyList")
		end
	-- otherwise let's check if we've changed screens, if so, let's clear the list of recently seen enemies
	else
		currScreenID = getMemory(0x3002BF6, 0x1)
		currRoomID = getMemory(0x3002C0C, 0x1) -- actually this is the ID of the last room entered, but this is what gets set to 0 in the game when moving outside
		if (currScreenID ~= lastScreenID) or (currRoomID ~= lastRoomID) then
			initializeSeenSprites()
			callJSFunctionv2(SECOND_WINDOW_ID, "clearEnemyList")
		end
	end
	lastGameMode = currGameMode
	lastScreenID = currScreenID
	lastRoomID = currRoomID
	
	-- some enemies have the same ID but look different depending on which world they're in, so we need to compensate for that
	-- theoretically this should work even when inside rooms/dungeons during standard game play
	local isDarkWorld = 0
	if currScreenID >= 0x40 and currScreenID < 0x80 then
		isDarkWorld = 1
	end
	
	-- now let's check the sprite tables and update the enemy bottom bar as needed
	local outputStr = ""
	local activeFlag = 0
	local spriteID = -1
	for i = 0, 0x10, 1 do
		activeFlag = getMemory(0x30031D2 + i, 1)
		if activeFlag == 0x9 then
			spriteID = getMemory(0x3003222 + i, 1)
			
			-- a lot of enemies have slight variations, so let's manually combine some of them
			if spriteID == 66 or spriteID == 68 or spriteID == 69 then	-- combining sword soldiers
				spriteID = 65
			elseif spriteID == 72 or spriteID == 73 then				-- spear soldiers
				spriteID = 67
			elseif spriteID == 71 then									-- bow soldiers
				spriteID = 70
			elseif spriteID == 86 then									-- zoras
				spriteID = 85
			elseif spriteID == 12 then									-- octoroks
				spriteID = 8
			elseif spriteID == 119 then									-- bubbles
				spriteID = 21
			elseif spriteID == 169 then									-- クリーピーs
				spriteID = 168
			elseif spriteID == 132 then									-- Eyegores
				spriteID = 131
			elseif spriteID == 36 then									-- Baris
				spriteID = 35
			elseif spriteID == 167 then									-- Stalfos
				spriteID = 133
			elseif spriteID == 74 and isDarkWorld == 1 then				-- Bombs from bomb-dropping/throwing enemies in the dark world don't need this
				goto endofloop
			end
			
			-- buzzblobs change into cukemans when you use magic powder on them, so we need to take that into account
			if spriteID == 13 then
				local variationID = getMemory(0x30032B2 + i, 1)
				if variationID == 1 then
					spriteID = 313
				end
			-- other enemies can be changed with magic powder, so this handles them hopefully
			elseif spriteID == 143 then
				-- if we're outside, then let's display them as Slimes, probably isn't foolproof though
				if currGameMode == 9 then
					spriteID = 443
				end
			-- floor tiles share the same number as swimmers?
			elseif spriteID == 148 then
				local variationID = getMemory(0x3003292 + i, 1)
				if variationID == 1 then
					spriteID = 448
				end
			-- sometimes enemy 0 gets picked up when an enemy is being deleted so let's ignore most of this by skipping enemy 0 when indoors
			elseif spriteID == 0 and currGameMode == 7 then
				goto endofloop
			end
			
			-- what is enemy 79? enemy 10? enemy 124? (decimal)
			-- test zoro (dust critters from wall)
			-- lots of other little things that could be done to improve:
			-- 		bari, when split, turn into biri, which share the same enemyID - need to differentiate them somehow?
			
			-- we'll display the current sprite on the sidebar only if we haven't seen it recently already
			if seenSprite[spriteID + 1] == false then
				seenSprite[spriteID + 1] = true
				callJSFunction_Int2v2(SECOND_WINDOW_ID, "appendEnemy", spriteID, isDarkWorld)
			end
			
			::endofloop::
		end
	end
	
	if outputStr ~= "" then
		writeHTMLv2(SECOND_WINDOW_ID, "bottom-container", outputStr)
	end
end


function getCurrentGameMode()
	return getMemory(0x3002BDC, 1)
end


-- ==================================================================================================
-- VARIOUS HTML WANDERBAR DISPLAY FUNCTIONS
-- ==================================================================================================

function displayScriptText(lineID, textType)
	if DO_AUTO_SCRIPTSTATS == true then	
		if textType == "script" then
			lineStatus[lineID] = "true"
			saveScriptStatus()
			updateScriptPercentageSeen()
		end
	end

	callJSFunctionv2(MAIN_WINDOW_ID, "clearAllData")
	callJSFunction_Str2v2(MAIN_WINDOW_ID, "displayAllTranslations", lineID, textType)
end

-- ==============================================================================================
-- BORING INTERNAL FUNCTIONS FOR KEEPING TRACK OF LINES SEEN SO FAR, ETC.
-- ==============================================================================================

function initScriptStatus()
	for i=1,totalLines,1
	do
		lineStatus[i] = "false"
	end
	
	if file_exists(DATA_ROOT .. SCRIPT_STATUS_FILE) == false then
		saveScriptStatus()
	end
end


function loadScriptStatus()
	seenLines = 0
	
	index = 1
	for line in io.lines(DATA_ROOT .. SCRIPT_STATUS_FILE) do
		lineStatus[index] = line
		index = index + 1
		
		if line == "true" then
			seenLines = seenLines + 1
		end
	end
end


function saveScriptStatus()
	seenLines = 0

	-- SAVE THE CURRENT SCRIPT VIEWING STATUS TO A DEDICATED FILE
	file = io.open(DATA_ROOT .. SCRIPT_STATUS_FILE, "w+")
	io.output(file)
	for k, v in pairs(lineStatus) do
		if v == "true" then
			seenLines = seenLines + 1
			io.write("true\n")
		else
			io.write("false\n")
		end
	end
	file:close()
	
	
	-- LET'S ALSO SAVE THE VIEWING DATA TO A HANDY .JS FILE FOR THE FINAL COMPLETION TALLY PAGE
	local tallyStr = ""
	local count = 0
	for k, v in pairs(lineStatus) do
	    if count > 0 then
			tallyStr = tallyStr .. ","
		end
		
		tallyStr = tallyStr .. v
		count = count + 1
	end
	tallyStr = "var flags = [" .. tallyStr .. "];"
	
	local width = math.sqrt(count)
	width = "var width = " .. math.floor(width) .. ";"
	
	local projectTitle = "var projectTitle = \"" .. PROJECT_TITLE .. "\";"
	
	file = io.open(DATA_ROOT .. SCRIPT_TALLY_FILE, "w+")
	io.output(file)
	io.write(projectTitle .. "\n")
	io.write(width .. "\n")
	io.write(tallyStr .. "\n\n")	
	file:close()
end


function updateScriptPercentageSeen()
	local rawPercent = seenLines / totalLines
	local percent = string.format("%.2f", rawPercent * 100)
	local str = "GBA Text Seen: " .. seenLines .. "/" .. totalLines .. " (" .. percent .. "%)"

	file = io.open(DATA_ROOT .. SCRIPT_PERCENT_FILE, "w+")
	io.output(file)
	io.write(str)
	file:close()
end


function file_exists(name)
	local file = io.open(name,"r")
	if file ~= nil then
		file:close()
		return true
	else
		return false
	end
end

function initializeSeenSprites()
	seenSprite = { }
	for i = 1, 0x202, 1 do
		seenSprite[i] = false
	end
end