dropTime = 100; // how fast text windows open and close in the sidebar

function appendEnemy(enemyID, isDarkWorld)
{
	var origEnemyID = enemyID;
	var splitExists = false;
	
	// some enemies have alternate forms, so we'll handle them by adding 300 to their original spriteIDs and then processing them here
	if (enemyID > 300)
	{
		enemyID = enemyID - 300;
		enemyID = enemyID + "-1";
	}

	// Get Japanese Super Famicom name	
	// if this isn't really an enemy let's just leave and not waste resources
	// but if we're dealing with a special enemy (ID above 300) then let's continue anyway
	var sfc = $("#sfc-enemy-" + enemyID).html();
	if (sfc == null || sfc == "" || sfc == "-")
	{
		return;
	}
	var tempName = sfc.split("|");
	sfc = tempName[0];
	if (isDarkWorld && tempName.length > 1)
	{
		sfc = tempName[1];
		splitExists = true;
	}
	
	// Get English Super NES name
	var snes = $("#snes-enemy-" + enemyID).html();
	if (!snes || snes == "" || snes == "-")
	{
		snes = "";
	}
	var tempName = snes.split("|");
	snes = tempName[0];
	if (isDarkWorld && tempName.length > 1)
	{
		snes = tempName[1];
		splitExists = true;
	}
	
	// Get English GBA name
	var gbaE = $("#gba-e-enemy-" + enemyID).html();
	if (!gbaE || gbaE == "" || gbaE == "-")
	{
		gbaE = "";
	}
	var tempName = gbaE.split("|");
	gbaE = tempName[0];
	if (isDarkWorld && tempName.length > 1)
	{
		gbaeE = tempName[1];
		splitExists = true;
	}
	
	// Get English Prima GBA name
	var prima = $("#prima-enemy-" + enemyID).html();
	if (!prima || prima == "" || prima == "-")
	{
		prima = "";
	}
	var tempName = prima.split("|");
	prima = tempName[0];
	if (isDarkWorld && tempName.length > 1)
	{
		prima = tempName[1];
		splitExists = true;
	}
	
	// Get English Link Between Worlds guide name
	var albwE = $("#albw-e-enemy-" + enemyID).html();
	if (!albwE || albwE == "" || albwE == "-")
	{
		albwE = "";
	}
	var tempName = albwE.split("|");
	albwE = tempName[0];
	if (isDarkWorld && tempName.length > 1)
	{
		albwE = tempName[1];
		splitExists = true;
	}
	

	// Now let's figure out what image to use with this enemy
	var image = enemyID + ".png";
	if (isDarkWorld && splitExists)
	{
		image = enemyID + "dw.png";
	}

	// Let's see if we should display this as a boss enemy
	var imageClass = "enemy-image";
	if (enemyID == 9 || enemyID == 82 || enemyID == 83 || enemyID == 84 || enemyID == 122 || enemyID == 136 || enemyID == 140 || enemyID == 146 || enemyID == 162 || enemyID == 189 || enemyID == 203 || enemyID == 206 || enemyID == 214)
	{
		imageClass = "boss-image";
	}

	
	// Now let's put together our enemy info table
	// theroetically it's possible an enemy info box could get overfilled with info but most of the time it probably won't happen...?
	var infoTable = "<table class=\"enemy-info-table\">";
	if (sfc)	{ infoTable += "<tr class=\"enemy-sfc\"><td>SFC:</td><td>" + sfc + "</td></tr>"; }
	if (snes)	{ infoTable += "<tr class=\"enemy-snes\"><td>SNES:</td><td>" + snes + "</td></tr>"; }
	if (gbaE)	{ infoTable += "<tr class=\"enemy-gba-e\"><td>GBA:</td><td>" + gbaE + "</td></tr>"; }
	if (prima)	{ infoTable += "<tr class=\"enemy-prima\"><td>Prima:</td><td>" + prima + "</td></tr>"; }
	if (albwE)	{ infoTable += "<tr class=\"enemy-albw-e\"><td>ALBW:</td><td>" + albwE + "</td></tr>"; }
	infoTable += "</table>";
	
	// Now let's display the completed enemy entry box
	//$("#bottom-container").prepend("<div class=\"enemy-id-" + enemyID + " sprite-info-box fade-in-and-scale\"><div class=\"" + imageClass + "\" style=\"background-image:url(images/enemies/" + image + ");\"></div><div style=\"position:absolute;top:0;left:0;\">" + enemyID + "</div>" + infoTable + "</div>");
	$("#bottom-container").prepend("<div class=\"enemy-id-" + enemyID + " sprite-info-box fade-in-and-scale\"><div class=\"" + imageClass + "\" style=\"background-image:url(images/enemies/" + image + ");\"></div>" + infoTable + "</div>");

}

// *********************************************************************************
// ESSENTIAL INTERNAL ROUTINES
// *********************************************************************************

// =================================================================================
// 
// =================================================================================
function replaceHtml(el, html) {
	var oldEl = typeof el === "string" ? document.getElementById(el) : el;
	var newEl = oldEl.cloneNode(false);
	newEl.innerHTML = html;
	oldEl.parentNode.replaceChild(newEl, oldEl);
	return newEl;
};


function writeHTML(id, msg)
{
	replaceHtml(id, msg);
}
	

// =================================================================================
// Copy the contents of one HTML element into another
// The destination's previous contents are overwritten
// =================================================================================
function copyHTMLElement(source, destination)
{
	$(destination).html($(source).html());
}


// =================================================================================
// Move the contents of one HTML element into another
// The source's current contents are removed
// The destination's previous contents are overwritten
// =================================================================================
function moveHTMLElement(source, destination)
{
	$(destination).html($(source).detach().html());
	//$(source).appendTo(destination);
	/*var s = $(source);
	var h = s.html();
	s.empty();
	$(destination).html(h);*/
	//$(source).detach().html(destination);
	//$(destination).show()
}


// =================================================================================
// Append content of source div to content of destination div
// Since this uses jQuery, IDs need to use #
// The copied HTML content is wrapped in a div for convenience purposes
// =================================================================================
function appendHTMLElement(source, destination)
{
	$(destination).append("<div class=\"" + $(source).attr("id") + "\">" + $(source).html() + "</div>");
}


// *********************************************************************************
// GAME-SPECIFIC ROUTINES
// *********************************************************************************


// =================================================================================
// A super-helpful way to clear the main viewing area of all text
// =================================================================================

function clearAllData()
{
	$(".scriptarea > div").html("");
	$(".scriptarea > div").show();
}




// =================================================================================
// A handy way to copy a div into the script area and have it display in a fancy way
// =================================================================================
//var scriptCloseTimer;
function displayScriptDiv(source, target, showInstantly)
{
	//window.clearTimeout(scriptCloseTimer);
	var s = $(source).html();
	if (s == null || s == "")
	{
		s = "<div class=\"notext\">-NO TEXT-</div>";
		dropTime = 0;
	}

	if (showInstantly)
	{
		$(target).html(s).show();
	}
	else
	{
		$(target).hide().html(s);
	}
}


// =================================================================================
// Close the script area in a fancy way
// =================================================================================
function closeScriptDiv()
{
	$("#scriptarea-gba-j").slideUp(dropTime);
	$("#scriptarea-sfc-j").slideUp(dropTime);
	$("#scriptarea-snes-e").slideUp(dropTime);

	$("#scriptarea-gba-j").removeClass("nodelay");
	$("#scriptarea-sfc-j").removeClass("nodelay");
	$("#scriptarea-snes-e").removeClass("nodelay");
	
	$("#scriptarea-gba-j-parent").removeClass("line-difference-found");
	$("#scriptarea-sfc-j-parent").removeClass("line-difference-found");
	$("#scriptarea-snes-e-parent").removeClass("line-difference-found");
	
}





// =================================================================================
// Display specified line number of each translation in each translation's box
// This includes a fancy opening animation best used for script text
// =================================================================================
function displayAllTranslations(lineNum, lineType)
{
	// WE NEED TO ACCOUNT FOR THE SLIGHTLY DIFFERENT LINE NUMBERS IN THE SFC AND SNES SCRIPTS
	var sfcLineNum = lineNum;
	if (sfcLineNum >= 6)
	{
		sfcLineNum -= 2;
	}
	if (sfcLineNum >= 14)
	{
		sfcLineNum -= 3;
	}
	else if (sfcLineNum >= 397)
	{
		sfcLineNum -= 8;
	}
	
	// SNES SCRIPT IS ONLY SLIGHTLY OFF FROM SFC SCRIPT
	var snesLineNum = sfcLineNum;
	if (snesLineNum >= 11)
	{
		snesLineNum += 2;
	}
	
	// some lines exist in the GBA version that don't have an equivalent in the SFC version
	if (lineNum >= 394 && lineNum <= 401)
	{
		sfcLineNum = 0;
		snesLineNum = 0;
	}
	// A BUNCH OF THE EARLY LINES ARE WEIRD AND NOT REALLY FIT FOR COMPARISON TEXT PURPOSES
	else if (lineNum <= 12)
	{
		sfcLineNum = 0;
		snesLineNum = 0;
	}
	
	$(".book-icon").remove();
	$("#scriptarea-gba-j-parent").removeClass("line-difference-found");
	$("#scriptarea-sfc-j-parent").removeClass("line-difference-found");
	$("#scriptarea-snes-e-parent").removeClass("line-difference-found");
	
	displayScriptDiv("#z3-gba-j-script-" + lineNum, "#scriptarea-gba-j", false);
	displayScriptDiv("#z3-sfc-j-script-" + sfcLineNum, "#scriptarea-sfc-j", false);
	displayScriptDiv("#z3-snes-e-script-" + snesLineNum, "#scriptarea-snes-e", false);

	$("#scriptarea-gba-j").slideDown(dropTime);
	$("#scriptarea-sfc-j").slideDown(dropTime);
	$("#scriptarea-snes-e").slideDown(dropTime);
	
	var textGBA_J = $("#z3-gba-j-script-" + lineNum).text();
	textGBA_J = textGBA_J.replace(" ", "");
	textGBA_J = textGBA_J.replace("　", "");
	
	var textGBA_E = $("#z3-gba-e-script-" + lineNum).text();
	textGBA_E = textGBA_E.replace(" ", "");
	textGBA_E = textGBA_E.replace("　", "");
	
	var textSFC_J = $("#z3-sfc-j-script-" + sfcLineNum).text();
	textSFC_J = textSFC_J.replace(" ", "");
	textSFC_J = textSFC_J.replace("　", "");
	
	var textSNES_E = $("#z3-snes-e-script-" + snesLineNum).text();
	textSNES_E = textSNES_E.replace(" ", "");
	textSNES_E = textSNES_E.replace("　", "");
	
	// FOR DEBUGGING PURPOSES
	//$("#scriptarea-snes-e").prepend("<div>GBA-E:" + lineNum + " / SFC-J: "+ sfcLineNum + " / SNES-E:" + snesLineNum + "</div>");
	
	if (textSNES_E != textGBA_E)
	{
		$("#scriptarea-snes-e").prepend("<a href=\"#\"><div class=\"book-icon\" comparisonheader=\"Super NES (E)|GBA (E)\" firstlinetype=\"snes-e\" firstlinenum=\"" + snesLineNum + "\" secondlinetype=\"gba-e\" secondlinenum=\"" + lineNum + "\"></div></a>");
	}
	
	if (textSFC_J != textGBA_J)
	{
		$("#scriptarea-gba-j").prepend("<a href=\"#\"><div class=\"book-icon\" comparisonheader=\"Super Famicom (J)|GBA (J)\" firstlinetype=\"sfc-j\" firstlinenum=\"" + sfcLineNum + "\" secondlinetype=\"gba-j\" secondlinenum=\"" + lineNum + "\"></div></a>");
	}
}



$(document).ready(function() {
	
	$("#container").on('click', '.book-icon', makeBookIconClickable);
});


clicks = 0;
function makeBookIconClickable()
{
	
	
	
	var firstlinetype = $(this).attr("firstlinetype");
		var firstlinenum = $(this).attr("firstlinenum");
		var secondlinetype = $(this).attr("secondlinetype");
		var secondlinenum = $(this).attr("secondlinenum");
		var comparisonheader = $(this).attr("comparisonheader");
		
		var firstline = $("#z3-" + firstlinetype + "-script-" + firstlinenum).html();
		var secondline = $("#z3-" + secondlinetype + "-script-" + secondlinenum).html();
		
		// we'll format the compared English text a little differently from the Japanese
		if (firstlinetype == "snes-e" || secondlinetype == "snes-e")
		{
			// convert all <br />s into |s temporarily
			/*firstline = firstline.split("<br><br>").join("|");
			firstline = firstline.split("<br>").join(" ");
			secondline = secondline.split("<br><br>").join("|");
			secondline = secondline.split("<br>").join(" ");*/
			//firstline = firstline.split("<br><br>").join("|");
			firstline = firstline.split("<br> ").join("@");
			firstline = firstline.split("<br>").join(" ");
			//secondline = secondline.split("<br><br>").join("|");
			secondline = secondline.split("<br> ").join("@");
			secondline = secondline.split("<br>").join(" ");
		}
		else
		{
			firstline = firstline.split("<br>").join("|");
			//firstline = firstline.split("<br> ").join("@");
			//firstline = firstline.split("<br>").join(" ");
			secondline = secondline.split("<br>").join("|");
			//secondline = secondline.split("<br> ").join("@");
			//secondline = secondline.split("<br>").join(" ");
		}

		var wikEdDiff = new WikEdDiff();
		var diffHtml = wikEdDiff.diff(firstline, secondline);
		diffHtml = diffHtml.split("|").join("<br>");
		diffHtml = diffHtml.split("@").join("<br> ");
		
		comparisonheaders = comparisonheader.split("|");
		comparisonheader = "<h2><span class=\"firstcompheader\">" + comparisonheaders[0] + "</span> ➔ <span class=\"secondcompheader\">" + comparisonheaders[1] + "</span></h2>";

		$.featherlight(comparisonheader + diffHtml);
		return false;
}


function clearEnemyList()
{
	$("#bottom-container").html("");
}
